@@ -7,6 +7,7 @@ import io
 from multiprocessing import Process
 from pyroute2.common import uuid32
 from pyroute2.common import debug
+from pyroute2.iocore import TimeoutError
 from pyroute2.netlink import NLMSG_CONTROL
 from pyroute2.netlink import NLMSG_TRANSPORT
 from pyroute2.netlink import IPRCMD_ACK
@@ -221,10 +222,28 @@ class IOCore(object):
         msg['attrs'] = attrs
         msg['header']['type'] = NLMSG_CONTROL
         msg.encode()
-        rsp = self.request(msg.buf.getvalue(),
-                           env_flags=NLT_CONTROL,
-                           nonce_pool=self.cmd_nonce,
-                           addr=addr)[0]
+
+        # try to send a command 3 times
+        for _ in range(3):
+            try:
+                rsp = self.request(msg.buf.getvalue(),
+                                   env_flags=NLT_CONTROL,
+                                   nonce_pool=self.cmd_nonce,
+                                   addr=addr)[0]
+            except Queue.Empty:
+                # if there is no reply, try again
+                # rsp variable will be initialized
+                continue
+
+            # if there is a reply, continue execution
+            # (break the loop)
+            break
+
+        else:
+            # if there was no break -- raise Queue.Empty()
+            # for compatibility
+            raise Queue.Empty()
+
         if rsp['cmd'] != IPRCMD_ACK:
             raise RuntimeError(rsp.get_attr('IPR_ATTR_ERROR'))
         if expect is not None:
@@ -301,15 +320,13 @@ class IOCore(object):
         for (uid, addr) in tuple(self.uids):
             try:
                 self.disconnect(uid, addr=addr)
-            except Queue.Empty as e:
+            except TimeoutError as e:
                 if addr == self.default_broker:
                     raise e
         self.command(IPRCMD_STOP)
         if self.forked_broker:
             self.forked_broker.join()
         self.ioloop.shutdown()
-        self.ioloop.join()
-
         self._brs.send(struct.pack('I', 4))
         self._brs.close()
         self.bridge.close()
@@ -413,8 +430,8 @@ class IOCore(object):
             # Bug-Url: http://bugs.python.org/issue1360
             try:
                 msg = queue.get(block=True, timeout=timeout)
-            except Queue.Empty as x:
-                e = x
+            except Queue.Empty:
+                e = TimeoutError()
                 if key == 0:
                     continue
                 else:
