@@ -141,6 +141,8 @@ RTM_SETNEIGHTBL = 67
 TC_H_INGRESS = 0xfffffff1
 TC_H_ROOT = 0xffffffff
 
+DEFAULT_TABLE = 254
+
 RTNL_GROUPS = RTNLGRP_IPV4_IFADDR |\
     RTNLGRP_IPV6_IFADDR |\
     RTNLGRP_IPV4_ROUTE |\
@@ -512,7 +514,7 @@ class IPRoute(Netlink):
         msg['family'] = family
         # you can specify the table here, but the kernel
         # will ignore this setting
-        table = kwarg.get('table', 0)
+        table = kwarg.get('table', DEFAULT_TABLE)
         msg['table'] = table if table <= 255 else 252
 
         # get a particular route
@@ -541,7 +543,7 @@ class IPRoute(Netlink):
     # removed due to redundancy. Only link shortcuts are left here for
     # now. Possibly, they should be moved to a separate module.
     #
-    def get_default_routes(self, family=AF_UNSPEC, table=None):
+    def get_default_routes(self, family=AF_UNSPEC, table=DEFAULT_TABLE):
         '''
         Get default routes
         '''
@@ -600,6 +602,25 @@ class IPRoute(Netlink):
         return [k['index'] for k in
                 [i for i in self.get_links() if 'attrs' in i] if
                 [l for l in k['attrs'] if l[0] == name and l[1] == value]]
+
+    def flush_routes(self, *argv, **kwarg):
+        '''
+        Flush routes -- purge route records from a table.
+        Arguments are the same as for `get_routes()`
+        routine. Actually, this routine implements a pipe from
+        `get_routes()` to `nlm_request()`.
+        '''
+        flags = NLM_F_ACK | NLM_F_CREATE | NLM_F_EXCL | NLM_F_REQUEST
+        ret = []
+        debug = self.debug
+        self.debug = True
+        kwarg['table'] = kwarg.get('table', DEFAULT_TABLE)
+        for route in self.get_routes(*argv, **kwarg):
+            ret.append(self.nlm_request(route,
+                                        msg_type=RTM_DELROUTE,
+                                        msg_flags=flags))
+        self.debug = debug
+        return ret
     # 8<---------------------------------------------------------------
 
     # 8<---------------------------------------------------------------
@@ -940,7 +961,8 @@ class IPRoute(Netlink):
                                 msg_flags=flags)
 
     def rule(self, command, table, priority=32000, rtype='RTN_UNICAST',
-             rtscope='RT_SCOPE_UNIVERSE', family=AF_INET, src=None):
+             rtscope='RT_SCOPE_UNIVERSE', family=AF_INET, src=None,
+             src_len=None, dst=None, dst_len=None, fwmark=None):
         '''
         Rule operations
 
@@ -954,6 +976,9 @@ class IPRoute(Netlink):
         * family   - rule's family (socket.AF_INET (default) or
                      socket.AF_INET6)
         * src      - IP source for Source Based (Policy Based) routing's rule
+        * dst      - IP for Destination Based (Policy Based) routing's rule
+        * src_len  - Mask for Source Based (Policy Based) routing's rule
+        * dst_len  - Mask for Destination Based (Policy Based) routing's rule
 
         Example::
             ip.rule('add', 10, 32000)
@@ -981,6 +1006,24 @@ class IPRoute(Netlink):
             ...
             32004: from 10.64.75.141 lookup 14
             ...
+
+        Example::
+            iproute.rule('add', 15, 32005, dst='10.64.75.141', dst_len=24)
+
+        Will create::
+            #ip ru sh
+            ...
+            32005: from 10.64.75.141/24 lookup 15
+            ...
+
+        Example::
+            iproute.rule('add', 15, 32006, dst='10.64.75.141', fwmark=10)
+
+        Will create::
+            #ip ru sh
+            ...
+            32006: from 10.64.75.141 fwmark 0xa lookup 15
+            ...
         '''
         if table < 0 or table > 254:
             raise 'unsupported table number'
@@ -997,14 +1040,26 @@ class IPRoute(Netlink):
         msg['scope'] = rtscopes[rtscope]
         msg['attrs'] = [['RTA_TABLE', table]]
         msg['attrs'].append(['RTA_PRIORITY', priority])
-        msg['dst_len'] = 0
-        msg['src_len'] = 0
+        if fwmark is not None:
+            msg['attrs'].append(['RTA_PROTOINFO', fwmark])
+        addr_len = {AF_INET6: 128, AF_INET:  32}[family]
+        if(dst_len is not None and dst_len >= 0 and dst_len <= addr_len):
+            msg['dst_len'] = dst_len
+        else:
+            msg['dst_len'] = 0
+        if(src_len is not None and src_len >= 0 and src_len <= addr_len):
+            msg['src_len'] = src_len
+        else:
+            msg['src_len'] = 0
         if src is not None:
             msg['attrs'].append(['RTA_SRC', src])
-            addr_len = {
-                AF_INET6: 128,
-                AF_INET:  32}[family]
-            msg['src_len'] = addr_len
+            if src_len is None:
+                msg['src_len'] = addr_len
+        if dst is not None:
+            msg['attrs'].append(['RTA_DST', dst])
+            if dst_len is None:
+                msg['dst_len'] = addr_len
+
         return self.nlm_request(msg, msg_type=command,
                                 msg_flags=msg_flags)
     # 8<---------------------------------------------------------------
